/*
    FreeRTOS V6.1.0 - Copyright (C) 2010 Real Time Engineers Ltd.

    ***************************************************************************
    *                                                                         *
    * If you are:                                                             *
    *                                                                         *
    *    + New to FreeRTOS,                                                   *
    *    + Wanting to learn FreeRTOS or multitasking in general quickly       *
    *    + Looking for basic training,                                        *
    *    + Wanting to improve your FreeRTOS skills and productivity           *
    *                                                                         *
    * then take a look at the FreeRTOS books - available as PDF or paperback  *
    *                                                                         *
    *        "Using the FreeRTOS Real Time Kernel - a Practical Guide"        *
    *                  http://www.FreeRTOS.org/Documentation                  *
    *                                                                         *
    * A pdf reference manual is also available.  Both are usually delivered   *
    * to your inbox within 20 minutes to two hours when purchased between 8am *
    * and 8pm GMT (although please allow up to 24 hours in case of            *
    * exceptional circumstances).  Thank you for your support!                *
    *                                                                         *
    ***************************************************************************

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/*
 * This is a very simple demo that demonstrates task and queue usages only.
 * Details of other FreeRTOS features (API functions, tracing features,
 * diagnostic hook functions, memory management, etc.) can be found on the
 * FreeRTOS web site (http://www.FreeRTOS.org) and in the FreeRTOS book.
 * Details of this demo (what it does, how it should behave, etc.) can be found
 * in the accompanying PDF application note.
 *
*/

/* Kernel includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "LPC17xx.h"


/* Own libs */
#include "led.h"
#include "printf.h"
#include "uart.h"
#include "timer.h"

/* Priorities at which the tasks are created. */
#define mainQUEUE_RECEIVE_TASK_PRIORITY		( tskIDLE_PRIORITY + 2 )
#define	mainQUEUE_SEND_TASK_PRIORITY		( tskIDLE_PRIORITY + 1 )

/* The bit of port 0 that the LPCXpresso LPC13xx LED is connected. */
#define mainLED_BIT 						( 22 )

/* The rate at which data is sent to the queue, specified in milliseconds. */
#define mainQUEUE_SEND_FREQUENCY_MS			( 100 / portTICK_RATE_MS )

/* The number of items the queue can hold.  This is 1 as the receive task
will remove items as they are added, meaning the send task should always find
the queue empty. */
#define mainQUEUE_LENGTH					( 1 )


/* Used as a loop counter to create a very crude delay. */
#define mainDELAY_LOOP_COUNT		( 0xfffff )

/* The task functions. */
void vTask1( void *pvParameters );
void vTask2( void *pvParameters );
void vTask3( void *pvParameters );
void vTask4( void *pvParameters );



/*
 * Simple function to toggle the LED on the LPCXpresso LPC17xx board.
 */
static void prvToggleLED( void );

/* The queue used by both tasks. */
static xQueueHandle xQueue = NULL;

static const char* msg = "TEST Log_log\n";

static char readBuffer[128];


/*-----------------------------------------------------------*/

int main(void)
{
    /* Initialise P0_22 for the LED. */
    Log_start(UART3,9600);
    Log_log(msg);
    /* Create one of the two tasks. */
    xTaskCreate(	vTask1,		/* Pointer to the function that implements the task. */
                    "Task 1",	/* Text name for the task.  This is to facilitate debugging only. */
                    240,		/* Stack depth in words. */
                    NULL,		/* We are not using the task parameter. */
                    1,			/* This task will run at priority 1. */
                    NULL );		/* We are not using the task handle. */
    /* Create the other task in exactly the same way. */
    /*xTaskCreate( vTask2, "Task 2", 240, NULL, 1, NULL );

    xTaskCreate( vTask3, "Task 3", 240, NULL, 1, NULL );*/
    xTaskCreate( vTask4, "Task 4", 240, NULL, 1, NULL );
    /* Start the tasks running. */
    vTaskStartScheduler();
    /* If all is well we will never reach here as the scheduler will now be
    running.  If we do reach here then it is likely that there was insufficient
    heap available for the idle task to be created. */
    for( ;; );
}
/*-----------------------------------------------------------*/



/*
 * This  prints through UART3 his task name and current tick counter
 * */
void vTask1( void *pvParameters )
{
    const char *pcTaskName = "Task 1 is running";
    volatile unsigned long ul;
    static uint32_t counter = 0;
    static char buffer[32];
    /* As per most tasks, this task is implemented in an infinite loop. */
    for( ;; )
    {
        /* Print out the name of this task. */
        vPrintString( pcTaskName );
        counter = xTaskGetTickCount();
        /* Send through UART3 pcTaskName string */
        sprintf(buffer, "%s %d\n",pcTaskName, counter);
        Log_log(buffer);
        /* Delay for a period. */
        for( ul = 0; ul < mainDELAY_LOOP_COUNT; ul++ )
        {
            /* This loop is just a very crude delay implementation.  There is
            nothing to do in here.  Later exercises will replace this crude
            loop with a proper delay/sleep function. */
        }
    }
}
/*-----------------------------------------------------------*/
/**
 * This task uses Log_log to log a text
 */
void vTask2( void *pvParameters )
{
    uint32_t counter = 0;
    const char *pcTaskName = "Task 2 is running\n";
    volatile unsigned long ul;
    /* As per most tasks, this task is implemented in an infinite loop. */
    for( ;; )
    {
        /* Print out the name of this task. */
        vPrintString( pcTaskName );
        Log_log(pcTaskName);
        /* Delay for a period. */
        for( ul = 0; ul < mainDELAY_LOOP_COUNT; ul++ )
        {
            /* This loop is just a very crude delay implementation.  There is
            nothing to do in here.  Later exercises will replace this crude
            loop with a proper delay/sleep function. */
        }
    }
}
/*-----------------------------------------------------------*/

/*
 * This  prints through UART3 his task name and current tick counter
 * */
void vTask3( void *pvParameters )
{
    uint32_t counter = 0;
    const char *pcTaskName = "Task 3 is running";
    volatile unsigned long ul;
    static char buffer[32];
    /* As per most tasks, this task is implemented in an infinite loop. */
    for( ;; )
    {
        /* Print out the name of this task. */
        vPrintString( pcTaskName );
        /* Send through UART3 pcTaskName string */
        sprintf(buffer, "%s %d\n",pcTaskName, counter);
        Log_log(buffer);
        /* Delay for a period. */
        for( ul = 0; ul < mainDELAY_LOOP_COUNT; ul++ )
        {
            /* This loop is just a very crude delay implementation.  There is
            nothing to do in here.  Later exercises will replace this crude
            loop with a proper delay/sleep function. */
        }
    }
}
/*-----------------------------------------------------------*/

/*
 * This  prints through UART3 his task name and current tick counter
 * */
void vTask4( void *pvParameters )
{
    uint32_t counter = 0;
    uint8_t buffer[64];
    const char *pcTaskName = "Task 4 is running\n";
    /* As per most tasks, this task is implemented in an infinite loop. */
    for( ;; )
    {
        /* Print out the name of this task. */
        Log_log(pcTaskName);
        int32_t ret = PrintF_scan(UART3, readBuffer, 1000);
        if( ret > 0 )
        {
            /* Send through UART3 pcTaskName string */
            sprintf(buffer, "Read %s\n",readBuffer);
            Log_log(buffer);
            memset(buffer,'\0',64);
        }
        else
        {
            Log_log("No data read\n");
        }
        Timer_sleepMs(TIMER0, 1000);
    }
}

/*-----------------------------------------------------------*/

static void prvToggleLED( void )
{
    Led_toggle();
}
/*-----------------------------------------------------------*/





void vApplicationIdleHook( void )
{
    /* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
    /* This example does not use the tick hook to perform any processing. */
}

void vApplicationMallocFailedHook( void )
{
    /* This function will only be called if an API call to create a task, queue
    or semaphore fails because there is too little heap RAM remaining. */
    for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
    /* This function will only be called if a task overflows its stack.  Note
    that stack overflow checking does slow down the context switch
    implementation. */
    for( ;; );
}
