/*
 * mma7736.c
 *
 *  Created on: May 5, 2015
 *      Author: yango
 */

#include <adc.h>
#include <stdio.h>

#include "ret.h"

#ifndef _BV
#define _BV(_x_) (1UL << (_x_))
#endif

volatile uint32_t ADCValue[ADC_NUM];
volatile uint32_t ADCIntDone = 0;

static xSemaphoreHandle lstMutex[ ADC_NUM ] ; /* array of mutex */
static int32_t isInitialized =  0;


/*****************************************************************************
** Function name:		ADCInit
**
** Descriptions:		initialize ADC channel
**
** parameters:
** Returned value:		None
**
*****************************************************************************/
void ADC_Init(  )
{
    uint32_t i, pclkdiv, pclk;
    /* Enable CLOCK into ADC controller */
    LPC_SC->PCONP |= (1 << 12);
    for ( i = 0; i < ADC_NUM; i++ )
    {
        ADCValue[i] = 0x0;
        if( 0 == isInitialized )
        {
            lstMutex[i] = xSemaphoreCreateMutex();
        }
    }
    isInitialized = 1;
    /* all the related pins are set to ADC inputs, AD0.0~7 */
    LPC_PINCON->PINSEL0 &= ~0x000000F0;	/* P0.2~3, A0.6~7, function 10 */
    LPC_PINCON->PINSEL0 |= 0x000000A0;
    LPC_PINCON->PINSEL1 &= ~0x003FC000;	/* P0.23~26, A0.0~3, function 01 */
    LPC_PINCON->PINSEL1 |= 0x00154000;
    LPC_PINCON->PINSEL3 |= 0xF0000000;	/* P1.30~31, A0.4~5, function 11 */
    /* No pull-up no pull-down (function 10) on these ADC pins. */
    LPC_PINCON->PINMODE0 &= ~0x000000F0;
    LPC_PINCON->PINMODE0 |= 0x000000A0;
    LPC_PINCON->PINMODE1 &= ~0x003FC000;
    LPC_PINCON->PINMODE1 |= 0x002A8000;
    LPC_PINCON->PINMODE3 &= ~0xF0000000;
    LPC_PINCON->PINMODE3 |= 0xA0000000;
    /* By default, the PCLKSELx value is zero, thus, the PCLK for
    all the peripherals is 1/4 of the SystemFrequency. */
    /* Bit 24~25 is for ADC */
    pclkdiv = (LPC_SC->PCLKSEL0 >> 24) & 0x03;
    switch ( pclkdiv )
    {
    case 0x00:
    default:
        pclk = SystemCoreClock/4;
        break;
    case 0x01:
        pclk = SystemCoreClock;
        break;
    case 0x02:
        pclk = SystemCoreClock/2;
        break;
    case 0x03:
        pclk = SystemCoreClock/8;
        break;
    }
    LPC_ADC->ADCR = ( 0x01 << 0 ) |  /* SEL=1,select channel 0~7 on ADC0 */
                    ( ( pclk  / ADC_CLK - 1 ) << 8 ) |  /* CLKDIV = Fpclk / ADC_Clk - 1 */
                    ( 0 << 16 ) | 		/* BURST = 0, no BURST, software controlled */
                    ( 0 << 17 ) |  		/* CLKS = 0, 11 clocks/10 bits */
                    ( 1 << 21 ) |  		/* PDN = 1, normal operation */
                    ( 0 << 24 ) |  		/* START = 0 A/D conversion stops */
                    ( 0 << 27 );		/* EDGE = 0 (CAP/MAT singal falling,trigger A/D conversion) */
    return;
}

/*****************************************************************************
** Function name:		ADCRead
**
** Descriptions:		Read ADC channel
**
** parameters:			Channel number
** Returned value:		Value read, if interrupt driven, return channel #
**
*****************************************************************************/
int32_t ADC_Read( uint8_t channelNum, uint32_t *outValue )
{
    if ( 1 != isInitialized )
    {
        return E_NON_INITIALIZED;
    }
    if (channelNum >= ADC_NUM)
    {
        return E_INCORRECT_ADC;
    }
    if( pdTRUE !=  xSemaphoreTake(lstMutex[channelNum], portMAX_DELAY) )
    {
        return E_CANNOT_MUTEX;
    }
    uint32_t regVal, ADC_Data;
    LPC_ADC->ADCR &= 0xFFFFFF00;
    LPC_ADC->ADCR |= (1 << 24) | (1 << channelNum);
    /* switch channel,start A/D convert */
    while ( 1 )			/* wait until end of A/D convert */
    {
        switch (channelNum)
        {
        case 0:
            regVal = LPC_ADC->ADDR0;
            break;
        case 1:
            regVal = LPC_ADC->ADDR1;
            break;
        case 2:
            regVal = LPC_ADC->ADDR2;
            break;
        case 3:
            regVal = LPC_ADC->ADDR3;
            break;
        case 4:
            regVal = LPC_ADC->ADDR4;
            break;
        case 5:
            regVal = LPC_ADC->ADDR5;
            break;
        case 6:
            regVal = LPC_ADC->ADDR6;
            break;
        case 7:
            regVal = LPC_ADC->ADDR7;
            break;
        }
        //regVal = LPC_ADC->DR[channelNum];
        /* read result of A/D conversion */
        if ( regVal & ADC_DONE )
        {
            break;
        }
    }
    LPC_ADC->ADCR &= 0xF8FFFFFF;	/* stop ADC now */
    if ( regVal & ADC_OVERRUN )	/* save data when it's not overrun, otherwise, return zero */
    {
        *outValue = 0;
        xSemaphoreGive(lstMutex[channelNum]);
        return E_ERROR;
    }
    ADC_Data = ( regVal >> 4 ) & 0xFFF;
    *outValue =  ADC_Data ;	/* return A/D conversion value */
    xSemaphoreGive(lstMutex[channelNum]);
    return RET_OK;
}

/*****************************************************************************
** Function name:		ADC0BurstRead
**
** Descriptions:		Use burst mode to convert multiple channels once.
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void ADCBurstRead( void )
{
    /* Start bits need to be zero before BURST mode can be set. */
    if ( LPC_ADC->ADCR & (0x7<<24) )
    {
        LPC_ADC->ADCR &= ~(0x7<<24);
    }
    LPC_ADC->ADCR &= ~0xFF;
    /* Read all channels, 0 through 7. */
    LPC_ADC->ADCR |= 0xFF;
    LPC_ADC->ADCR |= (0x1<<16);		/* Set burst mode and start A/D convert */
    return;						/* the ADC reading is done inside the
								handler, return 0. */
}
